# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from school_api.client.api.login import Login
from school_api.config import LOGIN_SESSION_SAVE_TIME


class LoginManagement(object):
    """ 登录会话管理 """
    _http = None
    account = None
    session = None
    base_url = None
    url_token = ''

    login = Login()

    def session_management(self):
        ''' 登录会话管理 '''
        login_session = None
        if self._get_login_session():
            session_expires_time = LOGIN_SESSION_SAVE_TIME - self._get_login_session_expires_time()

            if session_expires_time < 60 * 5:
                # 五分钟内，不检测登录会话是否过期
                login_session = self
            elif self.login.check_session():
                # 登录会话检测有效
                if session_expires_time < 60 * 10:
                    # 登录比较频繁的，更新会话时间 (例如：部门账号操作)
                    self.save_login_session()
                login_session = self
            else:
                # 会话过期, 删除会话
                self._del_login_session()

        return login_session

    def save_login_session(self):
        ''' 保存登录会话 '''
        key = self._get_login_session_key()
        cookie = self._http.cookies.get_dict()
        session = {
            "cookie": cookie,
            "url_token": self.url_token
        }
        self.session.set(key, session, LOGIN_SESSION_SAVE_TIME)

    def _get_login_session(self):
        ''' 获取登录会话 '''
        key = self._get_login_session_key()
        session = self.session.get(key)
        if not session:
            return False

        self.url_token = session['url_token']
        self._http.cookies.update(session['cookie'])
        return True

    def _del_login_session(self):
        ''' 删除登录会话 '''
        key = self._get_login_session_key()
        self.session.delete(key)
        self._http.cookies.clear()

    def _get_login_session_expires_time(self):
        """ 获取登录会话过期时间 """
        key = self._get_login_session_key()
        return self.session.expires_time(key)

    def _get_login_session_key(self):
        ''' 获取缓存登录会话的key '''
        key = '{}:{}:{}:{}'.format(self.school.code, 'login_session', self.base_url, self.user.account)
        return key



# -*- coding: utf-8 -*-
'''
    @Time       : 2016 - 2018
    @Author     : dairoot
    @Email      : 623815825@qq.com
    @description: 课表解析
'''
from __future__ import absolute_import, unicode_literals

import re
import six
from bs4 import BeautifulSoup


class BaseScheduleParse():
    ''' 课表页面解析模块 '''

    def __init__(self, html, time_list, schedule_type):
        self.schedule_year = ''
        self.schedule_term = ''
        self.time_list = time_list
        self.schedule_type = schedule_type
        self.schedule_list = [[], [], [], [], [], [], []]

        soup = BeautifulSoup(html, "html.parser")
        option_args = soup.find_all("option", {"selected": "selected"})
        if option_args:
            self.schedule_year = option_args[0].text
            self.schedule_term = option_args[1].text
            table = soup.find("table", {"id": "Table6"}) if \
                schedule_type == 1 else soup.find("table", {"id": "Table1"})
            trs = table.find_all('tr')
            self.html_parse(trs)

    def html_parse(self, trs):
        """
        :param n+1: 为周几
        :param i-1: 为第几节
        :param arr: ["课程", "时间", "姓名", "地点", "节数", "周数数组"]
        :param row_arr: 为周几第几节 的课程信息
        :param rowspan: 表示该课程有几节课
        :return:
        """
        pattern = r'^\([\u2E80-\u9FFF]{1,3}\d+\)'
        # 每天最多有10节课, 数据从2到12, (i-1) 代表是第几节课 (偶数节 不获取)
        for i in range(2, 12, 2):
            tds = trs[i].find_all("td")
            # 去除无用数据，比如(上午, 第一节...  等等)
            if i in [2, 6, 10]:
                tds.pop(0)
            tds.pop(0)
            # 默认获取7天内的课表(周一到周日) tds 长度为7
            for day, day_c in enumerate(tds):
                row_arr = []
                if day_c.text != u' ':
                    td_str = day_c.__unicode__()
                    rowspan = 2 if 'rowspan="2"' in td_str else 1
                    td_main = re.sub(r'<td align="Center".*?>', '', td_str)[:-5]

                    for text in td_main.split('<br/><br/>'):
                        course_arr = self._get_td_course_info(text)
                        if course_arr[0] and not re.match(pattern, course_arr[0]):
                            course_arr[1] = self._get_weeks_text(course_arr[1])
                            weeks_arr = self._get_weeks_arr(course_arr[1])
                            row_arr.append(course_arr + [rowspan, weeks_arr])
                self.schedule_list[day].append(row_arr)

    def get_schedule_dict(self):
        ''' 返回课表数据 字典格式 '''
        all_schedule = [[], [], [], [], [], [], []]
        color = ['green', 'blue', 'purple', 'red', 'yellow']

        for day, day_schedule in enumerate(self.schedule_list):
            for section, section_schedule in enumerate(day_schedule):
                section_schedule_dict = []
                color_index = (day * 3 + section + 1) % 5
                for schedule in section_schedule:
                    if schedule:
                        section_schedule_dict.append({
                            "color": color[color_index],
                            "name": schedule[0],
                            "weeks_text": schedule[1],
                            "teacher": schedule[2],
                            "place": schedule[3],
                            "section": schedule[4],
                            "weeks_arr": schedule[5],
                            "time": self.time_list[schedule[4]][section]
                        })
                all_schedule[day].append(section_schedule_dict)

        schedule_data = {
            'schedule_term': self.schedule_term,
            'schedule_year': self.schedule_year,
            'schedule': all_schedule
        }
        return schedule_data

    def _get_weeks_text(self, class_time):
        ''' 课程周数文本 '''
        if not self.schedule_type:
            weeks_text = re.findall(r"{(.*)}", class_time)[0]
        else:
            # 2节/周
            # 2节/单周(7-7)
            # 1-10,13-18(1,2)
            if '2节/' in class_time:
                weeks_text = class_time if '(' in class_time else class_time + '(1-18)'
            else:
                weeks_text = class_time.split('(')[0]
        return weeks_text

    @staticmethod
    def _get_weeks_arr(weeks_text):
        """
        将上课时间 转成 数组形式
        :param class_time: 上课时间
        :param weeks_text: 课程周数文本
        :param weeks_arr: 上课周数数组
        :return:
        """
        weeks_arr = []
        step = 2 if '单' in weeks_text or '双' in weeks_text else 1
        for split_text in weeks_text.split(','):
            weeks = re.findall(r'(\d{1,2})-(\d{1,2})', split_text)

            if weeks:
                weeks_arr += range(int(weeks[0][0]), int(weeks[0][1]) + 1, step)
            else:
                weeks_arr += [int(split_text)]

        return weeks_arr

    @staticmethod
    def _get_td_course_info(text):
        ''' 获取td标签的课程信息 '''
        text = re.sub(r'<[/]{0,1}font[^>]*?>', '', text)
        text = re.sub(r'^<br/>', '', text)

        if six.PY2:
            # 以下兼容 python2 版本解析处理
            text = re.sub(r'</br></br></br>$', '', text)
            text = text.replace('<br>', '<br/>')

        info_arr = []
        for k in text.split('<br/>'):
            if k not in ['选修', '公选', '必修', '限选', '任选']:
                info_arr.append(k)

        info_arr = info_arr[:4:]
        if len(info_arr) == 3:
            # 没有上课地点的情况
            info_arr.append('')
        return info_arr


class ScheduleParse(BaseScheduleParse):
    ''' 课表节数合并 '''

    def __init__(self, html, time_list, schedule_type=0):
        BaseScheduleParse.__init__(self, html, time_list, schedule_type)
        self.merger_same_schedule()

    def merger_same_schedule(self):
        """
        :param day_schedule: 一天的课程
        :param section_schedule: 一节课的课程
        :return:
        """
        for day_schedule in self.schedule_list:
            self._merger_day_schedule(day_schedule)

    def _merger_day_schedule(self, day_schedule):
        """
        将同一天相邻的相同两节课合并
        例如：[[["英语", "2节/双周(14-14)", "姓名", "1-301", "2", "[7,8]"],[...]],
        [["英语", "2节/双周(14-14)", "姓名", "1-301", "2", "[7,8]"],[...]]]
        合并为： 课程节数修改
        [[["英语", "2节/双周(14-14)", "姓名", "1-301", "4", "[7,8]"],[...]],
        [[...]]]
        """
        # 先合并 同一节课的相同课程
        for section_schedule in day_schedule:
            self._merger_section_schedule(section_schedule)

        # 再合并 同一天相邻的相同两节课合并
        day_slen = len(day_schedule)
        for i in range(day_slen - 1):
            for last_i, last_schedule in enumerate(day_schedule[i]):
                for next_i, next_schedule in enumerate(day_schedule[i + 1]):
                    if last_schedule and next_schedule:
                        # 课程名 上课地点 上课时间 教师名
                        if last_schedule[0] == next_schedule[0] and \
                            last_schedule[1] == next_schedule[1] and \
                                last_schedule[2] == next_schedule[2] and\
                                last_schedule[3] == next_schedule[3]:

                            day_schedule[i][last_i][4] += day_schedule[i + 1][next_i][4]
                            day_schedule[i + 1][next_i] = []

    @staticmethod
    def _merger_section_schedule(section_schedule):
        """
        将同一节课的相同课程合并
        例如：[["英语", "2节/单周(7-7)", "姓名", "1-301", "2", "[7]"],
         ["英语", "2节/双周(8-8)", "姓名", "1-301", "2", "[8]"]]
         合并为：课程时间修改
         [["英语", "2节/单周(7-7),2节/双周(8-8)", "姓名", "1-301", "2", "[7,8]"]]
        """
        section_slen = len(section_schedule)
        for i in range(section_slen):
            for j in range(i + 1, section_slen):
                if section_schedule[i] and section_schedule[j]:
                    # 课程名 一样时
                    if section_schedule[i][0] == section_schedule[j][0]:
                        # 并且上课时间不同，上课地点 一样时
                        if section_schedule[i][1] != section_schedule[j][1] and \
                                section_schedule[i][3] == section_schedule[j][3]:
                            section_schedule[j][5] += section_schedule[i][5]
                            section_schedule[j][1] += ',' + section_schedule[i][1]
                            section_schedule[i] = []

                        # 课程名和上课时间一样时 将上一个赋为空
                        if section_schedule[i] and section_schedule[i][1] == section_schedule[j][1]:
                            section_schedule[i] = []



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import six.moves.urllib.parse as urlparse
import time
from bs4 import BeautifulSoup
from requests import RequestException

from school_api.client.api.base import BaseSchoolApi
from school_api.client.api.utils.schedule_parse import ScheduleParse
from school_api.client.utils import ScheduleType
from school_api.exceptions import ScheduleException


class PlaceSchedule(BaseSchoolApi):
    schedule_url = None
    payload = None

    def get_schedule(self,
                     campus_list=None,
                     building_list=None,
                     classroom_type_list=None,
                     classroom_name_list=None,
                     filter_campus_list=None,
                     filter_building_list=None,
                     filter_classroom_type_list=None,
                     **kwargs):
        '''
        课表信息 获取入口
        生成器一旦报错则将退出：https://codeday.me/bug/20180125/124136.html
        除了解析异常其他报错均不抛出

        :param campus_list: 校区列表
        :param building_list: 楼号列表
        :param classroom_type_list: 教室类别列表
        :param classroom_name_list: 教室名称列表

        :param filter_campus_list: 排除校区列表
        :param filter_building_list: 排除楼号列表
        :param filter_classroom_type_list: 排除教室类别列表
        :param kwargs: requests 参数
        :yield: 生成器
        '''

        self.schedule_url = self.school_url['PLACE_SCHEDULE_URL'] + \
                            urlparse.quote(self.user.account.encode('gb2312'))

        if not self._update_payload(**kwargs):
            yield {'error': "获取教学场地课表失败"}
        else:
            # 遍历校区
            for campus in self.payload['campus_list']:
                if self._is_skip(campus["name"], campus_list, filter_name_list=filter_campus_list):
                    continue
                if not self._update_payload(campus, **kwargs):
                    continue

                # 遍历楼号
                for building in self.payload['building_list']:
                    kwargs['building'] = building
                    if self._is_skip(building["name"], building_list, filter_name_list=filter_building_list):
                        continue
                    if not self._update_payload(campus, **kwargs):
                        continue

                    # 遍历教室类别
                    for classroom_type in self.payload['classroom_type_list']:
                        kwargs['classroom_type'] = classroom_type
                        if self._is_skip(classroom_type["name"], classroom_type_list,
                                         filter_name_list=filter_classroom_type_list):
                            continue
                        if not self._update_payload(campus, **kwargs):
                            continue

                        # 遍历教室名称
                        for classroom_name in self.payload['classroom_name_list']:
                            if self._is_skip(classroom_name["name"], classroom_name_list):
                                continue

                            kwargs['classroom_name'] = classroom_name
                            # 请求接口获取课表数据
                            data = self._get_result(campus, **kwargs)
                            if data:
                                yield data

    def _get_result(self, campus, **kwargs):
        """ 处理请求结果，并返回 """
        try:
            res = self._get_api(campus, **kwargs)
        except ScheduleException:
            return

        schedule = ScheduleParse(res.text, self.time_list, ScheduleType.CLASS).get_schedule_dict()

        data = {
            "campus": campus["name"],
            "building": kwargs['building']["name"],
            "classroom_type": kwargs['classroom_type']["name"],
            "classroom_name": kwargs['classroom_name']["name"]
        }
        data.update(schedule)
        return data

    def _get_api(self, campus=None, **kwargs):
        """ 请求函数 """
        if self.payload and campus:
            building = kwargs.pop('building', None)
            classroom_type = kwargs.pop('classroom_type', None)
            classroom_name = kwargs.pop('classroom_name', None)
            data = {
                "Button1": "",
                "xq": self.payload['schedule_term'],
                "xn": self.payload['schedule_year'],
                "ddlXq": campus["value"],
                'ddllh': building["value"] if building else '',
                "ddlJslb": classroom_type["value"].encode('gb2312') if classroom_type else '',
                "ddlJsmc": classroom_name["value"].encode('gb2312') if classroom_name else '',
                "__VIEWSTATE": self.payload['view_state'],
            }
            _request = self._post
        else:
            data = ""
            _request = self._get

        try:
            res = _request(self.schedule_url, data=data, **kwargs)
        except RequestException:
            raise ScheduleException(self.code, '获取教学场地课表失败')
        return res

    def _update_payload(self, *args, **kwargs):
        # 更新提交参数 payload
        try:
            res = self._get_api(*args, **kwargs)
        except ScheduleException:
            return None

        try:
            self.payload = self._get_payload(res.text)
        except AttributeError:
            return None
        except Exception:
            time.sleep(2)
            return None
        return True

    @staticmethod
    def _get_payload(html):
        ''' 获取课表提交参数 '''
        pre_soup = BeautifulSoup(html, "html.parser")
        view_state = pre_soup.find(attrs={"name": "__VIEWSTATE"})['value']
        searchbox = pre_soup.find("div", {"class": "searchbox"})

        schedule_year = searchbox.find("select", {"id": "xn"}).find(
            "option", {"selected": "selected"}).text
        schedule_term = searchbox.find("select", {"id": "xq"}).find(
            "option", {"selected": "selected"}).text

        campuses = searchbox.find(id='ddlXq').find_all('option')
        buildings = searchbox.find(id='ddllh').find_all('option')
        types = searchbox.find(id='ddlJslb').find_all('option')
        names = searchbox.find(id='ddlJsmc').find_all('option')

        campuses = [{"name": v.text, "value": v['value']} for v in campuses]
        buildings = [{"name": v.text, "value": v['value']} for v in buildings]
        types = [{"name": v.text, "value": v['value']} for v in types]
        names = [{"name": v.text, "value": v['value']} for v in names]

        payload = {
            'view_state': view_state,
            'schedule_term': schedule_term,
            'schedule_year': schedule_year,
            'campus_list': campuses,
            'building_list': buildings,
            'classroom_type_list': types,
            'classroom_name_list': names,
        }
        return payload

    @staticmethod
    def _is_skip(name, name_list, filter_name_list=None):
        ''' 检查是否跳过 '''
        if (name_list and name not in name_list) or (filter_name_list and name in filter_name_list):
            return True
        return None



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals


class BaseSchoolApi(object):
    """ WeChat API base class """

    def __init__(self, client=None):
        self._client = client

    def _get(self, url, **kwargs):
        return self._client.get(url, **kwargs)

    def _post(self, url, **kwargs):
        return self._client.post(url, **kwargs)

    def _head(self, url, **kwargs):
        return self._client.head(url, **kwargs)

    def _get_view_state(self, url, **kwargs):
        return self._client.get_view_state(url, **kwargs)

    def _switch_proxy(self):
        return self._client.switch_proxy()

    def _update_url_token(self, url_token):
        return self._client.update_url_token(url_token)

    @property
    def code(self):
        return self._client.school.code

    @property
    def user(self):
        return self._client.user

    @property
    def school_url(self):
        return self._client.school.url_path_list[self.user.user_type]

    @property
    def time_list(self):
        return self._client.school.time_list
